#!/usr/bin/env python

# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.
#
# Copyright (c) 2016-2018 Intra2net AG <info@intra2net.com>

"""
.. note:: This unit tests cannot be run as a standalone module from the command line
    because it has internal autotest dependencies. It can however be easilty run
    from the test suite command line as a "setup" manual step.
"""
import unittest
import unittest.mock as mock
import subprocess

from src import arnied_wrapper


class DummyCmdOutputMapping(object):

    fail_switch = False
    cmds = [
        {"cmd": "pgrep -l -x arnied", "stdout": b"", "returncode": 0},
        {"cmd": 'echo "LICENSE_ACCEPTED,0: \\"1\\"" | set_cnf', "stdout": b"", "returncode": 0},
        {"cmd": '/usr/intranator/bin/arnied_helper --wait-for-program-end GENERATE', "stdout": b"", "returncode": 0},
        {"cmd": 'get_cnf PROVIDER 1', "stdout": b"PRODIVER 1, 'autotest'", "returncode": 0},
        {"cmd": 'tell-connd --online P1', "stdout": b"", "returncode": 0},
        {"cmd": 'get_cnf VIRSCAN_UPDATE_CRON  | set_cnf -x', "stdout": b"", "returncode": 0},
        {"cmd": '/usr/intranator/bin/arnied_helper --is-scheduled-or-running GENERATE', "stdout": b"", "returncode": 1},
        {"cmd": '/usr/intranator/bin/arnied_helper --is-scheduled-or-running GENERATE_OFFLINE', "stdout": b"", "returncode": 1},
        {"cmd": 'echo \'VIRSCAN_UPDATE_DNS_PUSH,0:"0"\' |set_cnf', "stdout": b"", "returncode": 0},
        {"cmd": 'rm -f /var/intranator/schedule/UPDATE_VIRSCAN_NODIAL*', "stdout": b"", "returncode": 0},
        {"cmd": '/usr/intranator/bin/arnied_helper --transfer-mail', "stdout": b"", "returncode": 0},
    ]
    asserted_cmds = []

    def __init__(self, cmd="", ignore_errors=False, vm=None):
        self.returncode = 0
        self.stdout = b""
        self._get_result(cmd)
        if self.fail_switch:
            self.returncode = 1

    def __str__(self):
        return "status %i, stdout %s" % (self.returncode, self.stdout)

    def _get_result(self, cmd):
        for dummy_cmd in self.asserted_cmds:
            if dummy_cmd['cmd'] == cmd:
                self.returncode = dummy_cmd['returncode']
                self.stdout = dummy_cmd['stdout']
                return
        raise ValueError("Could not locate the command '%s' among the known answers "
                         "for the universe" % cmd)


@mock.patch('src.arnied_wrapper.run_cmd', DummyCmdOutputMapping)
class ArniedWrapperTest(unittest.TestCase):

    def setUp(self):
        DummyCmdOutputMapping.fail_switch = False
        DummyCmdOutputMapping.asserted_cmds = []
        self.cmd_db = DummyCmdOutputMapping.cmds

    def test_verify_running(self):
        DummyCmdOutputMapping.asserted_cmds = self.cmd_db[0:1]
        arnied_wrapper.verify_running(timeout=1)
        DummyCmdOutputMapping.fail_switch = True
        with self.assertRaises(RuntimeError):
            arnied_wrapper.verify_running(timeout=1)

    def test_accept_license(self):
        DummyCmdOutputMapping.asserted_cmds = self.cmd_db[1:3]
        arnied_wrapper.accept_licence()
        # make sure an error is ignored since license might
        # already be accepted
        DummyCmdOutputMapping.fail_switch = True
        arnied_wrapper.accept_licence()

    def test_go_online(self):
        DummyCmdOutputMapping.asserted_cmds = self.cmd_db[3:5]
        arnied_wrapper.go_online(1)
        # TODO: for some reason failing to go online doesn't raise
        # an error which could be very misleading during debugging
        # DummyCmdOutputMapping.fail_switch = True
        # with self.assertRaises(Exception):
        #     arnied_wrapper.go_online(1)

    def test_disable_virscan(self):
        DummyCmdOutputMapping.asserted_cmds = self.cmd_db[5:10]
        arnied_wrapper.disable_virscan()

    def test_email_transfer(self):
        DummyCmdOutputMapping.asserted_cmds = self.cmd_db[10:11]
        arnied_wrapper.email_transfer()
